/******************************************************************************
** mpi_util.c
** MPI utility functions
** $Id: mpi_util.c,v 1.5 2005/06/20 04:57:34 koadman Exp $
** $Revision: 1.5 $
** 
** This file is a part of mpiBLAST and is copyright (c)
** by Aaron Darling of Los Alamos National Laboratory for the Regents
** of the University of California.
** 
** License:
** 
** Copyright 2002-2005. The Regents of the University of California.
** 
** This material was produced under U.S. Government contract
** W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
** by the University of California for the U.S. Department of Energy.
** The Government is granted for itself and other acting on its behalf
** a paid-up, non-exclusive, irrevocable worldwide license in the
** material to reproduce, prepare derivative works, and perform
** publicly and display publicly. Beginning five (5) years after
** November 6, 2002, subject to additional five-year worldwide
** renewals, the Government is granted for itself and others acting on
** its behalf a paid-up, nonexclusive, irrevocable worldwide license
** in this material to reproduce, prepare derivative works, distribute
** copies to the public, perform publicly and display publicly, and to
** permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
** STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
** ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
** ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
** COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
** OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
** PRIVATELY OWNED RIGHTS.
** 
** Additionally, this program is free software; you can distribute it
** and/or modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2 of
** the License, or any later version. Accordingly, this program is
** distributed in the hope that it will be useful, but WITHOUT ANY
** WARRANTY; without even the implied warranty of MERCHANTABILITY or
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
** License for more details. You should have received a copy of the
** GNU General Public License along with this program; if not, write
** to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA 02111-1307, USA.
** 
** ------------------------------------------------------------
** 
** See the file 'COPYING' for the details of the GNU General Public License
** 
******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "mpi_util.h"
#include <stdlib.h>

/* a simple wrapper around MPE_Log_event that consolidates
 * all the #ifdefs in one place
 */
void meta_MPE_Log_event( int a, int b, const char* c )
{
#ifdef MPE
	MPE_Log_event( a, b, c );
#endif
}

/* a simple wrapper around meta_MPI_Alloc_mem that calls malloc() if
 * meta_MPI_Alloc_mem isn't available
 */
int meta_MPI_Alloc_mem( MPI_Aint size, MPI_Info info, void* baseptr )
{
#ifdef HAVE_MPI_ALLOC_MEM
	return MPI_Alloc_mem( size, info, baseptr );
#else
	*(void**)baseptr = malloc(size);
	return 0;
#endif
}

int meta_MPI_Free_mem( void* base )
{
#ifdef HAVE_MPI_ALLOC_MEM
	return MPI_Free_mem(base);
#else
	free(base);
	return 0;
#endif
}


