/******************************************************************************
** mpiblast_cleanup.cpp
** A tool to remove mpiblast-related data from compute nodes
** $Id: mpiblast_cleanup.cpp,v 1.4 2005/06/20 03:58:52 koadman Exp $
** $Revision: 1.4 $
** 
** This file is a part of mpiBLAST and is copyright (c)
** by Aaron Darling of Los Alamos National Laboratory for the Regents
** of the University of California.
** 
** License:
** 
** Copyright 2002-2005. The Regents of the University of California.
** 
** This material was produced under U.S. Government contract
** W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
** by the University of California for the U.S. Department of Energy.
** The Government is granted for itself and other acting on its behalf
** a paid-up, non-exclusive, irrevocable worldwide license in the
** material to reproduce, prepare derivative works, and perform
** publicly and display publicly. Beginning five (5) years after
** November 6, 2002, subject to additional five-year worldwide
** renewals, the Government is granted for itself and others acting on
** its behalf a paid-up, nonexclusive, irrevocable worldwide license
** in this material to reproduce, prepare derivative works, distribute
** copies to the public, perform publicly and display publicly, and to
** permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
** STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
** ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
** ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
** COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
** OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
** PRIVATELY OWNED RIGHTS.
** 
** Additionally, this program is free software; you can distribute it
** and/or modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2 of
** the License, or any later version. Accordingly, this program is
** distributed in the hope that it will be useful, but WITHOUT ANY
** WARRANTY; without even the implied warranty of MERCHANTABILITY or
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
** License for more details. You should have received a copy of the
** GNU General Public License along with this program; if not, write
** to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA 02111-1307, USA.
** 
** ------------------------------------------------------------
** 
** See the file 'COPYING' for the details of the GNU General Public License
** 
******************************************************************************/
#include <mpi.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>

#include "mpiblast_config.hpp"
#include "mpiblast_util.hpp"
#include "fragment_list.hpp"
#include "file_util.hpp"

using namespace std;

int main( int argc, char* argv[]){
	//Default to .ncbirc, else 1st arg is mpiblast config-file
	//Does NOT work with 1st arg == /path/to/.ncbirc
	string config_filename = ".ncbirc";
	if( argc != 1 && argc != 2 ){
		cerr << "Usage: " << argv[0] << " [/path/to/mpiblast.conf]" << endl;
		return -1;
	}
	if( argc == 2 )
		config_filename = argv[1];

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);

	MpiBlastConfig config( config_filename );

		// don't delete shared storage!!
		if (config.localPath() == config.sharedPath()){
			if( rank == 0 ){
				cerr << "Shared and local storage are identical.\n";
				cerr << "This program will not delete files from shared storage\n";
			}
			MPI_Finalize();
			return -1;
		}


	try {
		vector< string > arg_vector;
		arg_vector.push_back( argv[0] );
		initNCBI( arg_vector );
	
		deleteMpiBlastFiles( config.localPath() );
	}
	catch (char const* message){
		cerr << message << endl;
	}
	MPI_Finalize();
	return 0;
}
