/******************************************************************************
** blast_hooks.h
** Interface to the NCBI blastall.c functionality
** $Id: blast_hooks.h,v 1.17 2005/06/20 03:58:51 koadman Exp $
** $Revision: 1.17 $
** 
** This file is a part of mpiBLAST and is copyright (c)
** by Aaron Darling of Los Alamos National Laboratory for the Regents
** of the University of California.
** 
** License:
** 
** Copyright 2002-2005. The Regents of the University of California.
** 
** This material was produced under U.S. Government contract
** W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
** by the University of California for the U.S. Department of Energy.
** The Government is granted for itself and other acting on its behalf
** a paid-up, non-exclusive, irrevocable worldwide license in the
** material to reproduce, prepare derivative works, and perform
** publicly and display publicly. Beginning five (5) years after
** November 6, 2002, subject to additional five-year worldwide
** renewals, the Government is granted for itself and others acting on
** its behalf a paid-up, nonexclusive, irrevocable worldwide license
** in this material to reproduce, prepare derivative works, distribute
** copies to the public, perform publicly and display publicly, and to
** permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
** STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
** ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
** ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
** COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
** OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
** PRIVATELY OWNED RIGHTS.
** 
** Additionally, this program is free software; you can distribute it
** and/or modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2 of
** the License, or any later version. Accordingly, this program is
** distributed in the hope that it will be useful, but WITHOUT ANY
** WARRANTY; without even the implied warranty of MERCHANTABILITY or
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
** License for more details. You should have received a copy of the
** GNU General Public License along with this program; if not, write
** to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA 02111-1307, USA.
** 
** ------------------------------------------------------------
** 
** See the file 'COPYING' for the details of the GNU General Public License
** 
******************************************************************************/
#ifndef __mpiblastall_h__
#define __mpiblastall_h__

#ifdef __cplusplus
extern "C"{
#endif

#include <stdlib.h>

#ifndef Int2
#define Int2 short
#endif

#ifndef Int4
#define Int4 int
#endif

#ifndef Nlm_FloatHi
#define	Nlm_FloatHi double
#endif

#ifndef Boolean
#define	Boolean  unsigned char
#endif

#ifndef Uint1
#define	Uint1  unsigned char
#endif

#ifdef USE_NCBI_ASCII
const static int use_binary_asn = FALSE;
#else
const static int use_binary_asn = TRUE;
#endif

	/** Reads command line and initializes BLAST data structures */
Int2 initBLAST ();
/** Outputs an HTML header IF the HTML command line option was set */
void outputHtmlHeader();
/** Outputs the results in <code>sappy</code> */
Int2 outputResults( SeqAlignPtr sappy, int queryI );
/** Outputs an HTML footer IF the HTML command line option was set */
void outputHTMLfooter();
/** Cleans up BLAST data structures */
void cleanupBLAST();


/** Load query sequences from the multi-FastA file */
int loadQueries( void );
/** Free memory used to store the query sequences */
void cleanupQueries( void );

void MpiBlastEnableBioseqFetch();
void MpiBlastDisableBioseqFetch();

/** 
 * runBLAST executes BLAST in one of two modes:
 * SEARCH_MODE Executes BLAST and stores successful result ids in result_id_list 
 * COLLECT_STATS_MODE Prepares all data structures and performs calculations related
 *                    to BLAST without actually performing the search
 * When first_query == last_query only a single query is processed
 * to process all queries, set first_query = 0 and last_query = query_count - 1
 * @param mode	The mode to run BLAST in, as described above
 * @param first_query	The first query to process
 * @param last_query	The last query to process
 */
Int2 runBLAST( int mode, int first_query, int last_query );
extern BLAST_OptionsBlkPtr options;
extern ValNodePtr result_id_list;
extern ValNodePtr result_bsp_list;	/**< List of ByteStorePtrs, each of which contains all results (SeqAligns+Bioseqs) for a query */
extern int debug;			/**< print debugging messages when set to true */
extern int resume_run;  /**<choose append to outputfile when true */ 
extern Int8* query_adj_array;  /**< Array of query length adjustments (one per query) */
extern Int8* db_len_array;     /**< Array of effective database lengths (one per query) */
extern Int8** stats_array;		/**< Array of per-query search statistics */
extern Boolean strict_output_conformance;	/**< Set to true if the Altschul paper reference must be given instead of the mpiBLAST paper reference */
const static int SEARCH_MODE = 0;
const static int COLLECT_STATS_MODE = 1;

/** stub for Bioseq* functions.  Can't directly include edutil.h
  * for some reason.  */
NLM_EXTERN Boolean LIBCALL BioseqDelete( SeqIdPtr target, Int4 from, Int4 to, 
					   Boolean do_feat, Boolean do_split );

NLM_EXTERN Boolean LIBCALL BioseqInsert( SeqIdPtr from_id, Int4 from, Int4 to, Uint1 strand, SeqIdPtr to_id, Int4 pos, 
					   Boolean from_feat, Boolean to_feat, Boolean do_split );

NLM_EXTERN BioseqPtr LIBCALL BioseqCopy (SeqIdPtr newid, SeqIdPtr sourceid, Int4 from, Int4 to,
                               Uint1 strand, Boolean do_feat);

static AsnType biginttype = {BIGINT_TYPE, "BIGINT" ,0,16,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL};
static AsnType set_type = {313, "SET" ,0,17,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL}; 
static AsnType len_type = {0, "len" ,128,6,0,0,0,0,0,0,NULL,&biginttype,NULL,0,NULL};
static AsnType lenset_type = {402, "Subject-len" ,1,0,0,0,0,1,0,0,NULL,&set_type,&len_type,0,NULL};
static AsnType statdata_type = {0, "stat-data" ,128,6,0,0,0,0,0,0,NULL,&biginttype,NULL,0,NULL};
static AsnType statset_type = {402, "Stat-data" ,1,0,0,0,0,1,0,0,NULL,&set_type,&statdata_type,0,NULL};

#ifdef __cplusplus
}
#endif

#endif /* __mpiblastall_h__ */
